#!/usr/bin/env python3




##################################################
## DEPENDENCIES
import sys
import os
import os.path
try:
    import builtins as builtin
except ImportError:
    import __builtin__ as builtin
from os.path import getmtime, exists
import time
import types
from Cheetah.Version import MinCompatibleVersion as RequiredCheetahVersion
from Cheetah.Version import MinCompatibleVersionTuple as RequiredCheetahVersionTuple
from Cheetah.Template import Template
from Cheetah.DummyTransaction import *
from Cheetah.NameMapper import NotFound, valueForName, valueFromSearchList, valueFromFrameOrSearchList
from Cheetah.CacheRegion import CacheRegion
import Cheetah.Filters as Filters
import Cheetah.ErrorCatchers as ErrorCatchers
from Cheetah.compat import unicode
from xpdeint.Features._Feature import _Feature

##################################################
## MODULE CONSTANTS
VFFSL=valueFromFrameOrSearchList
VFSL=valueFromSearchList
VFN=valueForName
currentTime=time.time
__CHEETAH_version__ = '3.3.2'
__CHEETAH_versionTuple__ = (3, 3, 2, 'final', 0)
__CHEETAH_src__ = 'HaltNonFinite.tmpl'
__CHEETAH_srcLastModified__ = 'Mon, 17 Jul 2023 19:11:41 -0300'
__CHEETAH_docstring__ = 'Autogenerated by Cheetah: The Python-Powered Template Engine'

if __CHEETAH_versionTuple__ < RequiredCheetahVersionTuple:
    raise AssertionError(
      'This template was compiled with Cheetah version'
      ' %s. Templates compiled before version %s must be recompiled.'%(
         __CHEETAH_version__, RequiredCheetahVersion))

##################################################
## CLASSES

class HaltNonFinite(_Feature):

    ##################################################
    ## CHEETAH GENERATED METHODS


    def __init__(self, *args, **KWs):

        super(HaltNonFinite, self).__init__(*args, **KWs)
        if not self._CHEETAH__instanceInitialized:
            cheetahKWArgs = {}
            allowedKWs = 'searchList namespaces filter filtersLib errorCatcher'.split()
            for k,v in KWs.items():
                if k in allowedKWs: cheetahKWArgs[k] = v
            self._initCheetahInstance(**cheetahKWArgs)
        

    def description(self, **KWS):



        ## Generated from @def description: Halt simulation on non-finite results at line 27, col 1.
        trans = KWS.get("trans")
        if (not trans and not self._CHEETAH__isBuffering and not callable(self.transaction)):
            trans = self.transaction # is None unless self.awake() was called
        if not trans:
            trans = DummyTransaction()
            _dummyTrans = True
        else: _dummyTrans = False
        write = trans.response().write
        SL = self._CHEETAH__searchList
        _filter = self._CHEETAH__currentFilter
        
        ########################################
        ## START - generated method body
        
        write('''Halt simulation on non-finite results''')
        
        ########################################
        ## END - generated method body
        
        return _dummyTrans and trans.response().getvalue() or ""
        

    def postSingleStep(self, dict, **KWS):



        ## CHEETAH: generated from @def postSingleStep($dict) at line 31, col 1.
        trans = KWS.get("trans")
        if (not trans and not self._CHEETAH__isBuffering and not callable(self.transaction)):
            trans = self.transaction # is None unless self.awake() was called
        if not trans:
            trans = DummyTransaction()
            _dummyTrans = True
        else: _dummyTrans = False
        write = trans.response().write
        SL = self._CHEETAH__searchList
        _filter = self._CHEETAH__currentFilter
        
        ########################################
        ## START - generated method body
        
        # 
        integrator = dict['caller']
        write('''// Check if a component of any integration vector is non-finite
// no need to check all components since they will be quickly mixed
''')
        # 
        write('''if (''')
        separator = ''
        for integrationVector in integrator.integrationVectors: # generated from line 39, col 3
            suffix = ''
            if integrationVector.type == 'complex': # generated from line 41, col 5
                #  Check the real component
                suffix = '.Re()'
            _v = VFFSL(SL,"separator",True) # '${separator}' on line 45, col 1
            if _v is not None: write(_filter(_v, rawExpr='${separator}')) # from line 45, col 1.
            write('''_xmds_isnonfinite(_active_''')
            _v = VFFSL(SL,"integrationVector.id",True) # '${integrationVector.id}' on line 45, col 39
            if _v is not None: write(_filter(_v, rawExpr='${integrationVector.id}')) # from line 45, col 39.
            write('''[0]''')
            _v = VFFSL(SL,"suffix",True) # '${suffix}' on line 45, col 65
            if _v is not None: write(_filter(_v, rawExpr='${suffix}')) # from line 45, col 65.
            write(''')''')
            separator = ' || '
        write(''') {
  // One of the integration vectors has gone non-finite.
  // Sample any moment groups that have pending samples,
  // then leave the integrator.
  _LOG(_WARNING_LOG_LEVEL, "WARNING: halt_non_finite: Integration halted at ''')
        _v = VFFSL(SL,"integrator.propagationDimension",True) # '${integrator.propagationDimension}' on line 52, col 77
        if _v is not None: write(_filter(_v, rawExpr='${integrator.propagationDimension}')) # from line 52, col 77.
        write(''' = %e.\\n"
                           "         Non-finite number in integration vector in segment ''')
        _v = VFFSL(SL,"integrator.segmentNumber",True) # '${integrator.segmentNumber}' on line 53, col 89
        if _v is not None: write(_filter(_v, rawExpr='${integrator.segmentNumber}')) # from line 53, col 89.
        write('''.\\n", ''')
        _v = VFFSL(SL,"propagationDimension",True) # '$propagationDimension' on line 53, col 122
        if _v is not None: write(_filter(_v, rawExpr='$propagationDimension')) # from line 53, col 122.
        write(''');
  
  ''')
        _v = VFFSL(SL,"integrator.earlyTerminationCode",True) # '${integrator.earlyTerminationCode, autoIndent=True}' on line 55, col 3
        if _v is not None: write(_filter(_v, autoIndent=True, rawExpr='${integrator.earlyTerminationCode, autoIndent=True}')) # from line 55, col 3.
        write('''}
''')
        # 
        
        ########################################
        ## END - generated method body
        
        return _dummyTrans and trans.response().getvalue() or ""
        

    def writeBody(self, **KWS):



        ## CHEETAH: main method generated for this template
        trans = KWS.get("trans")
        if (not trans and not self._CHEETAH__isBuffering and not callable(self.transaction)):
            trans = self.transaction # is None unless self.awake() was called
        if not trans:
            trans = DummyTransaction()
            _dummyTrans = True
        else: _dummyTrans = False
        write = trans.response().write
        SL = self._CHEETAH__searchList
        _filter = self._CHEETAH__currentFilter
        
        ########################################
        ## START - generated method body
        
        write('''
''')
        # 
        # HaltNonFinite.tmpl
        # 
        # Created by Graham Dennis on 2008-05-04.
        # 
        # Copyright (c) 2008-2012, Graham Dennis
        # 
        # This program is free software: you can redistribute it and/or modify
        # it under the terms of the GNU General Public License as published by
        # the Free Software Foundation, either version 2 of the License, or
        # (at your option) any later version.
        # 
        # This program is distributed in the hope that it will be useful,
        # but WITHOUT ANY WARRANTY; without even the implied warranty of
        # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        # GNU General Public License for more details.
        # 
        # You should have received a copy of the GNU General Public License
        # along with this program.  If not, see <http://www.gnu.org/licenses/>.
        # 
        # Based on code originally written by Gabriel McManus
        write('''

''')
        
        ########################################
        ## END - generated method body
        
        return _dummyTrans and trans.response().getvalue() or ""
        
    ##################################################
    ## CHEETAH GENERATED ATTRIBUTES


    _CHEETAH__instanceInitialized = False

    _CHEETAH_version = __CHEETAH_version__

    _CHEETAH_versionTuple = __CHEETAH_versionTuple__

    _CHEETAH_src = __CHEETAH_src__

    _CHEETAH_srcLastModified = __CHEETAH_srcLastModified__

    featureName = 'HaltNonFinite'

    uselib = ['safe_math']

    _mainCheetahMethod_for_HaltNonFinite = 'writeBody'

## END CLASS DEFINITION

if not hasattr(HaltNonFinite, '_initCheetahAttributes'):
    templateAPIClass = getattr(HaltNonFinite,
                               '_CHEETAH_templateClass',
                               Template)
    templateAPIClass._addCheetahPlumbingCodeToClass(HaltNonFinite)


# CHEETAH was developed by Tavis Rudd and Mike Orr
# with code, advice and input from many other volunteers.
# For more information visit https://cheetahtemplate.org/

##################################################
## if run from command line:
if __name__ == '__main__':
    from Cheetah.TemplateCmdLineIface import CmdLineIface
    CmdLineIface(templateObj=HaltNonFinite()).run()


